unit Asteroid;

(*********************************************
TAsteroid->TPolygonSprite
*********************************************)

interface

uses
  SysUtils, WinTypes, WinProcs, Classes, Graphics, Controls, WinGPnt, Sprite,
  LkupTabl, Polygon, Utility;

type

  TAsteroidSize = ( asBig, asMedium, asSmall );

  TAsteroid = class( TPolygonSprite )
  private
     FSize: TAsteroidSize;
  protected
     procedure AddVertices( const nCol: byte ); override;
     function GetCollision: integer;
     function GetScore: integer;
  public
     property CollisionDistance: integer read GetCollision;
     property Score: integer read GetScore;
     property Size: TAsteroidSize read FSize;
     constructor CreateAsteroid( nSize: TAsteroidSize; lk: TLookupTable );
  end;

implementation

uses
  Main;

constructor TAsteroid.CreateAsteroid( nSize: TAsteroidSize; lk: TLookupTable );
begin
  FSize := nSize;
  inherited CreatePolygonSprite( Random( 8 ) + 225, lk );
  case FSize of
     asBig:
        Speed := Random( 10 ) + 15;
     asMedium:
        Speed := Random( 15 ) + 10;
     asSmall:
        Speed := Random( 25 );
  end;
  SpinSpeed := Random( 10 );
  MotionType := mtContinuous;
  Spin := TRotation( Random( 2 ) + 1 );
end;

(*********************************************
All TPolygonSprites must override this method.
It defines the "shape" of the sprite.  In this
case, the values are entered in the TStringHolder
on the form and we read them in and convert them.
*********************************************)
procedure TAsteroid.AddVertices( const nCol: byte );
var
  str: TStrings;
  sItem: string;
  pt: TPoint;

  procedure GetPoints( const sSection: string; nX, nY: integer );
  var
     i: integer;
  begin
     frmMain.strPoints.AssignSection( sSection, str );
     for i := 0 to str.Count - 1 do
        begin
           sItem := str[i];
           pt.X := StrToInt( GetToken( sItem, ',' ) ) - nX;
           pt.Y := StrToInt( sItem ) - nY;
           lstVertices.Add( TVertex.Create( pt, nCol ) );
        end;
  end;

begin
  str := TStringList.Create;
  case FSize of
     asBig:
        GetPoints( 'Big', 30, 20 );
     asMedium:
        GetPoints( 'Medium', 16, 16 );
     asSmall:
        GetPoints( 'Small', 13, 14 );
  end;
  str.Free;
end;

(*********************************************
The score gained when an asteroid is destroyed.
*********************************************)
function TAsteroid.GetScore: integer;
begin
  case FSize of
     asBig:
        Result := 50;
     asMedium:
        Result := 100;
     asSmall:
        Result := 250;
  end;
end;

(*********************************************
The distance required for a collision with
an asteroid.
*********************************************)
function TAsteroid.GetCollision: integer;
begin
  case FSize of
     asBig:
        Result := 18;
     asMedium:
        Result := 14;
     asSmall:
        Result := 9;
  end;
end;

end.
