unit Missile;

(*********************************************
TMissile->TSprite

This sprite descends directly from TSprite, and
has its own custom Render method that simply
sets pixels in the video buffer.  This is a re-
usable sprite class that is not specific to any
app.
*********************************************)

interface

uses
  SysUtils, WinTypes, WinProcs, Classes, Graphics, Controls, WinGPnt,
  Sprite, DIB, DIBSprit, Utility;

type

  TMissile = class( TSprite )
  private
     nLife: word;
     nColor: byte;
     nMaxLife: word;
  protected
  public
     constructor CreateMissile( nMax: word; nColorOffset: byte );
     procedure Move; override;
     procedure Render( winGPnt: TWinGPaintBox ); override;
  end;

implementation

constructor TMissile.CreateMissile( nMax: word; nColorOffset: byte );
begin
  inherited Create;
  nLife := 0;
  nColor := nColorOffset;
  Priority := 1;
  Speed := 1;
  nMaxLife := nMax;
  MotionType := mtContinuous;
end;

procedure TMissile.Move;
begin
  Inc( nLife );
  if nLife = nMaxLife then
     Kill;
  inherited Move;
end;

procedure TMissile.Render( winGPnt: TWinGPaintBox );
var
  n: byte;
begin
  n := Random( 16 ) + nColor;
  with ptPhysical, winGPnt do
     begin
        if ( X - 2 >= 0 ) and ( Y - 2 >= 0 ) and
           ( X + 2 < PhysicalWidth ) and ( Y + 2 < PhysicalHeight ) then
           begin
              Pixels[X, Y] := n;
              Pixels[X, Y - 1] := n;
              Pixels[X, Y + 1] := n;
              Pixels[X - 1, Y] := n;
              Pixels[X + 1, Y] := n;
              Pixels[X - 1, Y - 1] := n;
              Pixels[X - 1, Y + 1] := n;
              Pixels[X + 1, Y - 1] := n;
              Pixels[X + 1, Y + 1] := n;
              Pixels[X, Y - 2] := n;
              Pixels[X, Y + 2] := n;
              Pixels[X - 2, Y] := n;
              Pixels[X + 2, Y] := n;
           end;
     end;
end;

end.
